@@ -69,7 +69,7 @@ typedef struct tiledb_array_schema_evolution_t tiledb_array_schema_evolution_t;
  */
 TILEDB_EXPORT int32_t tiledb_array_schema_evolution_alloc(
     tiledb_ctx_t* ctx,
-    tiledb_array_schema_evolution_t** array_schema_evolution);
+    tiledb_array_schema_evolution_t** array_schema_evolution) TILEDB_NOEXCEPT;
 
 /**
  * Destroys an array schema evolution, freeing associated memory.
@@ -83,7 +83,7 @@ TILEDB_EXPORT int32_t tiledb_array_schema_evolution_alloc(
  * @param array_schema_evolution The array schema evolution to be destroyed.
  */
 TILEDB_EXPORT void tiledb_array_schema_evolution_free(
-    tiledb_array_schema_evolution_t** array_schema_evolution);
+    tiledb_array_schema_evolution_t** array_schema_evolution) TILEDB_NOEXCEPT;
 
 /**
  * Adds an attribute to an array schema evolution.
@@ -105,7 +105,7 @@ TILEDB_EXPORT void tiledb_array_schema_evolution_free(
 TILEDB_EXPORT int32_t tiledb_array_schema_evolution_add_attribute(
     tiledb_ctx_t* ctx,
     tiledb_array_schema_evolution_t* array_schema_evolution,
-    tiledb_attribute_t* attribute);
+    tiledb_attribute_t* attribute) TILEDB_NOEXCEPT;
 
 /**
  * Drops an attribute to an array schema evolution.
@@ -126,10 +126,13 @@ TILEDB_EXPORT int32_t tiledb_array_schema_evolution_add_attribute(
 TILEDB_EXPORT int32_t tiledb_array_schema_evolution_drop_attribute(
     tiledb_ctx_t* ctx,
     tiledb_array_schema_evolution_t* array_schema_evolution,
-    const char* attribute_name);
+    const char* attribute_name) TILEDB_NOEXCEPT;
 
 /**
  * Sets timestamp range in an array schema evolution
+ * This function sets the output timestamp of the committed array schema after
+ * evolution. The lo and hi values are currently required to be the same or else
+ * an error is thrown.
  *
  * **Example:**
  *
@@ -141,24 +144,44 @@ TILEDB_EXPORT int32_t tiledb_array_schema_evolution_drop_attribute(
  *
  * @param ctx The TileDB context.
  * @param array_schema_evolution The schema evolution.
- * @param attribute_name The name of the attribute to be dropped.
+ * @param lo The lower bound of timestamp range.
+ * @param hi The upper bound of timestamp range, it must euqal to the lower
+ * bound.
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_array_schema_evolution_set_timestamp_range(
     tiledb_ctx_t* ctx,
     tiledb_array_schema_evolution_t* array_schema_evolution,
     uint64_t lo,
-    uint64_t hi);
+    uint64_t hi) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*          ARRAY SCHEMA             */
 /* ********************************* */
 
+/**
+ * Gets timestamp range in an array schema evolution
+ *
+ * **Example:**
+ *
+ * @code{.c}
+ * uint64_t timestamp_lo = 0;
+ * uint64_t timestamp_hi = 0;
+ * tiledb_array_schema_evolution_timestamp_range(ctx,
+ * array_schema_evolution, &timestamp_lo, &timestamp_hi);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param array_schema_evolution The schema evolution.
+ * @param lo The lower bound of timestamp range.
+ * @param hi The upper bound of timestamp range.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
 TILEDB_EXPORT int32_t tiledb_array_schema_timestamp_range(
     tiledb_ctx_t* ctx,
     tiledb_array_schema_t* array_schema,
     uint64_t* lo,
-    uint64_t* hi);
+    uint64_t* hi) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*               ARRAY               */
@@ -182,7 +205,7 @@ TILEDB_EXPORT int32_t tiledb_array_schema_timestamp_range(
 TILEDB_EXPORT int32_t tiledb_array_evolve(
     tiledb_ctx_t* ctx,
     const char* array_uri,
-    tiledb_array_schema_evolution_t* array_schema_evolution);
+    tiledb_array_schema_evolution_t* array_schema_evolution) TILEDB_NOEXCEPT;
 
 /**
  * Upgrades an array to the latest format version.
@@ -201,7 +224,9 @@ TILEDB_EXPORT int32_t tiledb_array_evolve(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_array_upgrade_version(
-    tiledb_ctx_t* ctx, const char* array_uri, tiledb_config_t* config);
+    tiledb_ctx_t* ctx,
+    const char* array_uri,
+    tiledb_config_t* config) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*               QUERY               */
@@ -217,7 +242,7 @@ TILEDB_EXPORT int32_t tiledb_subarray_add_point_ranges(
     tiledb_subarray_t* subarray,
     uint32_t dim_idx,
     const void* start,
-    uint64_t count);
+    uint64_t count) TILEDB_NOEXCEPT;
 
 /**
  * Adds a set of point ranges along subarray dimension index. Each value
@@ -248,7 +273,7 @@ TILEDB_DEPRECATED_EXPORT int32_t tiledb_query_add_point_ranges(
     tiledb_query_t* query,
     uint32_t dim_idx,
     const void* start,
-    uint64_t count);
+    uint64_t count) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*        QUERY STATUS DETAILS       */
@@ -293,7 +318,7 @@ struct tiledb_query_status_details_t {
 TILEDB_EXPORT int32_t tiledb_query_get_status_details(
     tiledb_ctx_t* ctx,
     tiledb_query_t* query,
-    tiledb_query_status_details_t* status);
+    tiledb_query_status_details_t* status) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*              CONTEXT              */
@@ -332,7 +357,45 @@ TILEDB_EXPORT int32_t tiledb_query_get_status_details(
  * @return `TILEDB_OK` for success and `TILEDB_OOM` or `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_ctx_alloc_with_error(
-    tiledb_config_t* config, tiledb_ctx_t** ctx, tiledb_error_t** error);
+    tiledb_config_t* config,
+    tiledb_ctx_t** ctx,
+    tiledb_error_t** error) TILEDB_NOEXCEPT;
+
+/* ********************************* */
+/*           CONSOLIDATION           */
+/* ********************************* */
+
+/**
+ * Consolidates the given fragment URIs into a single fragment.
+ *
+ * Note: This API needs to be used with caution until we implement
+ * consolidation with timestamps. For now, if the non-empty domain of the
+ * consolidated fragments overlap anything in the fragments that come in
+ * between, this could lead to unpredictable behavior.
+ *
+ * **Example:**
+ *
+ * @code{.c}
+
+ * const char* uris[2]={"__0_0_0807b1428b6c4ff48b3cdb3283ca7903_10",
+ *                      "__1_1_d9d965753d224194965575c1e9cdeeda_10"};
+ * tiledb_array_consolidate(ctx, "my_array", uris, 2);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param array_uri The name of the TileDB array whose metadata will
+ *     be consolidated.
+ * @param fragment_uris URIs of the fragments to consolidate.
+ * @param num_fragments Number of URIs to consolidate.
+ *
+ * @return `TILEDB_OK` on success, and `TILEDB_ERR` on error.
+ */
+TILEDB_EXPORT int32_t tiledb_array_consolidate_fragments(
+    tiledb_ctx_t* ctx,
+    const char* array_uri,
+    const char** fragment_uris,
+    const uint64_t num_fragments,
+    tiledb_config_t* config) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*                GROUP              */
@@ -354,7 +417,9 @@ TILEDB_EXPORT int32_t tiledb_ctx_alloc_with_error(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_alloc(
-    tiledb_ctx_t* ctx, const char* group_uri, tiledb_group_t** group);
+    tiledb_ctx_t* ctx,
+    const char* group_uri,
+    tiledb_group_t** group) TILEDB_NOEXCEPT;
 
 /**
  * Opens a TileDB group. The group is opened using a query type as input.
@@ -386,7 +451,9 @@ TILEDB_EXPORT int32_t tiledb_group_alloc(
  *       the group object before opening the group.
  */
 TILEDB_EXPORT int32_t tiledb_group_open(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, tiledb_query_type_t query_type);
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_query_type_t query_type) TILEDB_NOEXCEPT;
 
 /**
  * Closes a TileDB group.
@@ -408,7 +475,7 @@ TILEDB_EXPORT int32_t tiledb_group_open(
  *     no effect.
  */
 TILEDB_EXPORT int32_t
-tiledb_group_close(tiledb_ctx_t* ctx, tiledb_group_t* group);
+tiledb_group_close(tiledb_ctx_t* ctx, tiledb_group_t* group) TILEDB_NOEXCEPT;
 
 /**
  * Creates a new TileDB group.
@@ -423,7 +490,7 @@ tiledb_group_close(tiledb_ctx_t* ctx, tiledb_group_t* group);
  *
  * @param group The TileDB group to be freed
  */
-TILEDB_EXPORT void tiledb_group_free(tiledb_group_t** group);
+TILEDB_EXPORT void tiledb_group_free(tiledb_group_t** group) TILEDB_NOEXCEPT;
 
 /**
  * Sets the group config.
@@ -449,7 +516,9 @@ TILEDB_EXPORT void tiledb_group_free(tiledb_group_t** group);
  * @note The config should be set before opening an group.
  */
 TILEDB_EXPORT int32_t tiledb_group_set_config(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, tiledb_config_t* config);
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_config_t* config) TILEDB_NOEXCEPT;
 
 /**
  * Gets the group config.
@@ -468,7 +537,9 @@ TILEDB_EXPORT int32_t tiledb_group_set_config(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_get_config(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, tiledb_config_t** config);
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_config_t** config) TILEDB_NOEXCEPT;
 
 /**
  * It puts a metadata key-value item to an open group. The group must
@@ -493,7 +564,7 @@ TILEDB_EXPORT int32_t tiledb_group_put_metadata(
     const char* key,
     tiledb_datatype_t value_type,
     uint32_t value_num,
-    const void* value);
+    const void* value) TILEDB_NOEXCEPT;
 
 /**
  * It deletes a metadata key-value item from an open group. The group must
@@ -510,7 +581,7 @@ TILEDB_EXPORT int32_t tiledb_group_put_metadata(
  *     (i.e., the function will not error out).
  */
 TILEDB_EXPORT int32_t tiledb_group_delete_metadata(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, const char* key);
+    tiledb_ctx_t* ctx, tiledb_group_t* group, const char* key) TILEDB_NOEXCEPT;
 
 /**
  * It gets a metadata key-value item from an open group. The group must
@@ -536,7 +607,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_metadata(
     const char* key,
     tiledb_datatype_t* value_type,
     uint32_t* value_num,
-    const void** value);
+    const void** value) TILEDB_NOEXCEPT;
 
 /**
  * It gets then number of metadata items in an open group. The group must
@@ -548,7 +619,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_metadata(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_get_metadata_num(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, uint64_t* num);
+    tiledb_ctx_t* ctx, tiledb_group_t* group, uint64_t* num) TILEDB_NOEXCEPT;
 
 /**
  * It gets a metadata item from an open group using an index.
@@ -575,7 +646,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_metadata_from_index(
     uint32_t* key_len,
     tiledb_datatype_t* value_type,
     uint32_t* value_num,
-    const void** value);
+    const void** value) TILEDB_NOEXCEPT;
 
 /**
  * Checks whether a key exists in metadata from an open group. The group must
@@ -595,7 +666,7 @@ TILEDB_EXPORT int32_t tiledb_group_has_metadata_key(
     tiledb_group_t* group,
     const char* key,
     tiledb_datatype_t* value_type,
-    int32_t* has_key);
+    int32_t* has_key) TILEDB_NOEXCEPT;
 
 /**
  * Add a member to a group
@@ -623,7 +694,7 @@ TILEDB_EXPORT int32_t tiledb_group_add_member(
     tiledb_group_t* group,
     const char* uri,
     const uint8_t relative,
-    const char* name);
+    const char* name) TILEDB_NOEXCEPT;
 
 /**
  * Remove a member from a group
@@ -642,7 +713,7 @@ TILEDB_EXPORT int32_t tiledb_group_add_member(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_remove_member(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, const char* uri);
+    tiledb_ctx_t* ctx, tiledb_group_t* group, const char* uri) TILEDB_NOEXCEPT;
 
 /**
  * Get the count of members in a group
@@ -667,7 +738,7 @@ TILEDB_EXPORT int32_t tiledb_group_remove_member(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_get_member_count(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, uint64_t* count);
+    tiledb_ctx_t* ctx, tiledb_group_t* group, uint64_t* count) TILEDB_NOEXCEPT;
 
 /**
  * Get a member of a group by index and details of group
@@ -707,7 +778,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_member_by_index(
     uint64_t index,
     char** uri,
     tiledb_object_t* type,
-    char** name);
+    char** name) TILEDB_NOEXCEPT;
 
 /**
  * Get a member of a group by index and details of group
@@ -745,7 +816,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_member_by_name(
     tiledb_group_t* group,
     const char* name,
     char** uri,
-    tiledb_object_t* type);
+    tiledb_object_t* type) TILEDB_NOEXCEPT;
 
 /**
  * Checks if the group is open.
@@ -756,7 +827,7 @@ TILEDB_EXPORT int32_t tiledb_group_get_member_by_name(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_is_open(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, int32_t* is_open);
+    tiledb_ctx_t* ctx, tiledb_group_t* group, int32_t* is_open) TILEDB_NOEXCEPT;
 
 /**
  * Retrieves the URI the group was opened with. It outputs an error
@@ -768,7 +839,9 @@ TILEDB_EXPORT int32_t tiledb_group_is_open(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_get_uri(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, const char** group_uri);
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    const char** group_uri) TILEDB_NOEXCEPT;
 
 /**
  * Retrieves the query type with which the group was opened.
@@ -789,7 +862,9 @@ TILEDB_EXPORT int32_t tiledb_group_get_uri(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_group_get_query_type(
-    tiledb_ctx_t* ctx, tiledb_group_t* group, tiledb_query_type_t* query_type);
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_query_type_t* query_type) TILEDB_NOEXCEPT;
 
 /**
  * Dump a string representation of a group
@@ -805,8 +880,165 @@ TILEDB_EXPORT int32_t tiledb_group_dump_str(
     tiledb_ctx_t* ctx,
     tiledb_group_t* group,
     char** dump_ascii,
-    uint8_t recursive);
+    const uint8_t recursive) TILEDB_NOEXCEPT;
 
+/* ********************************* */
+/*                FILESTORE          */
+/* ********************************* */
+
+/**
+ * Creates an array schema based on the properties of the provided URI
+ * or a default schema if no URI is provided
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_array_schema_t* schema;
+ * tiledb_filestore_schema_create(ctx, "/path/file.pdf", &schema);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param uri The file URI.
+ * @param array_schema The TileDB array schema to be created
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_schema_create(
+    tiledb_ctx_t* ctx,
+    const char* uri,
+    tiledb_array_schema_t** array_schema) TILEDB_NOEXCEPT;
+
+/**
+ * Imports a file into a TileDB filestore array
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_array_schema_t* schema;
+ * tiledb_filestore_schema_create(ctx, path_to_file, &schema);
+ * tiledb_array_create(ctx, path_to_array, schema);
+ * tiledb_filestore_uri_import(ctx, path_to_array, path_to_file,
+ * TILEDB_MIME_AUTODETECT);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param filestore_array_uri The array URI.
+ * @param file_uri The file URI.
+ * @param mime_type The mime type of the file
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_uri_import(
+    tiledb_ctx_t* ctx,
+    const char* filestore_array_uri,
+    const char* file_uri,
+    tiledb_mime_type_t mime_type) TILEDB_NOEXCEPT;
+
+/**
+ * Exports a filestore array into a bare file
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_filestore_uri_export(ctx, path_to_file, path_to_array);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param uri The file URI.
+ * @param uri The array URI.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_uri_export(
+    tiledb_ctx_t* ctx,
+    const char* file_uri,
+    const char* filstore_array_uri) TILEDB_NOEXCEPT;
+
+/**
+ * Writes size bytes starting at address buf into filestore array
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_array_schema_t* schema;
+ * tiledb_filestore_schema_create(ctx, NULL, &schema);
+ * tiledb_array_create(ctx, path_to_array, schema);
+ * tiledb_filestore_buffer_import(ctx, path_to_array, buf, size,
+ * TILEDB_MIME_AUTODETECT);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param uri The array URI.
+ * @param buf The input buffer
+ * @param size Number of bytes to be imported
+ * @param mime_type The mime type of the data
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_buffer_import(
+    tiledb_ctx_t* ctx,
+    const char* filestore_array_uri,
+    void* buf,
+    size_t size,
+    tiledb_mime_type_t mime_type) TILEDB_NOEXCEPT;
+
+/**
+ * Dump the content of a filestore array into a buffer
+ * **Example:**
+ *
+ * @code{.c}
+ * size_t size = 1024;
+ * void *buf = malloc(size);
+ * tiledb_filestore_buffer_export(ctx, path_to_array, 0, buf, size);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param filestore_array_uri The array URI.
+ * @param offset The offset at which we should start exporting from the array
+ * @param buf The buffer that will contain the filestore array content
+ * @param size The number of bytes to be exported into the buffer
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_buffer_export(
+    tiledb_ctx_t* ctx,
+    const char* filestore_array_uri,
+    size_t offset,
+    void* buf,
+    size_t size) TILEDB_NOEXCEPT;
+
+/**
+ * Get the uncompressed size of a filestore array
+ * **Example:**
+ *
+ * @code{.c}
+ * size_t size;
+ * tiledb_filestore_size(ctx, path_to_array, &size);
+ * void *buf = malloc(size);
+ * tiledb_filestore_buffer_export(ctx, path_to_array, 0, buf, size);
+ * free(buf);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param uri The array URI.
+ * @param size The returned uncompressed size of the filestore array
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_filestore_size(
+    tiledb_ctx_t* ctx,
+    const char* filestore_array_uri,
+    size_t* size) TILEDB_NOEXCEPT;
+
+/**
+ * Get the string representation of a mime type enum
+ *
+ * @param mime_type The mime enum
+ * @param str The resulted string representation
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_mime_type_to_str(
+    tiledb_mime_type_t mime_type, const char** str) TILEDB_NOEXCEPT;
+
+/**
+ * Turn a string mime type into a TileDB enum
+ *
+ * @param str The mime type string
+ * @param mime_type The resulted mime enum
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_mime_type_from_str(
+    const char* str, tiledb_mime_type_t* mime_type) TILEDB_NOEXCEPT;
 #ifdef __cplusplus
 }
 #endif
